@@ -33,7 +33,7 @@ extern "C" {
 namespace filter
 {
   
-  enum FILTER_TYPE { median=0, var=1 , min=2, max=3, sum=4, mean=5, minmax=6, dilate=7, erode=8, close=9, open=10, homog=11, sobelx=12, sobely=13, sobelxy=14, sobelyx=-14, smooth=15, density=16, mode=17, mixed=18, smoothnodata=19, threshold=20, ismin=21, ismax=22, heterog=23, order=24, stdev=25, dwt=26, dwti=27, dwt_cut=28, dwt_cut_from=29};
+  enum FILTER_TYPE { median=0, var=1 , min=2, max=3, sum=4, mean=5, minmax=6, dilate=7, erode=8, close=9, open=10, homog=11, sobelx=12, sobely=13, sobelxy=14, sobelyx=-14, smooth=15, density=16, mode=17, mixed=18, smoothnodata=19, threshold=20, ismin=21, ismax=22, heterog=23, order=24, stdev=25, dwt=26, dwti=27, dwt_cut=28, dwt_cut_from=29, savgolay=30};
 
    enum PADDING { symmetric=0, replicate=1, circular=2, constant=3};
 
@@ -65,9 +65,11 @@ public:
   void setTaps(const std::vector<double> &taps, bool normalize=true);
   void pushClass(short theClass=1){m_class.push_back(theClass);};
   void pushMask(short theMask=0){m_mask.push_back(theMask);};
+  int pushNoDataValue(double noDataValue=0);//{m_mask.push_back(theMask);};
   template<class T> void filter(const std::vector<T>& input, std::vector<T>& output);
   template<class T> void filter(const std::vector<T>& input, std::vector<T>& output, const std::string& method, int dim);
   template<class T> void smooth(const std::vector<T>& input, std::vector<T>& output, short dim);
+  template<class T> void smoothNoData(const std::vector<T>& input, const std::string& interpolationType, std::vector<T>& output);
   template<class T> void filter(T* input, int inputSize, std::vector<T>& output);
   template<class T> void smooth(T* input, int inputSize, std::vector<T>& output, short dim);
   //template<class T> void morphology(const std::vector<T>& input, std::vector<T>& output, const std::string& method, int dim, bool verbose=false);
@@ -75,7 +77,12 @@ public:
   void filter(const ImgReaderGdal& input, ImgWriterGdal& output);
   void stat(const ImgReaderGdal& input, ImgWriterGdal& output, const std::string& method);
   void filter(const ImgReaderGdal& input, ImgWriterGdal& output, const std::string& method, int dim);
+  void getSavGolayCoefficients(std::vector<double> &c, int np, int nl, int nr, int ld, int m);
+  void ludcmp(std::vector<double> &a, std::vector<int> &indx, double &d);
+  void lubksb(std::vector<double> &a, std::vector<int> &indx, std::vector<double> &b);
+  /* void savgolay(const ImgReaderGdal& input, ImgWriterGdal& output, int np, int nl, int nr, int m); */
   void smooth(const ImgReaderGdal& input, ImgWriterGdal& output, short dim);
+  void smoothNoData(const ImgReaderGdal& input, const std::string& interpolationType, ImgWriterGdal& output);
   double getCentreWavelength(const std::vector<double> &wavelengthIn, const Vector2d<double>& srf, const std::string& interpolationType, double delta=1.0, bool verbose=false);
   template<class T> double applySrf(const std::vector<double> &wavelengthIn, const std::vector<T>& input, const Vector2d<double>& srf, const std::string& interpolationType, T& output, double delta=1.0, bool normalize=false, bool verbose=false);
   template<class T> double applySrf(const std::vector<double> &wavelengthIn, const Vector2d<T>& input, const Vector2d<double>& srf, const std::string& interpolationType, std::vector<T>& output, double delta=1.0, bool normalize=false, int down=1, bool transposeInput=false, bool verbose=false);
@@ -125,6 +132,7 @@ private:
     m_filterMap["heterog"]=filter::heterog;
     m_filterMap["order"]=filter::order;
     m_filterMap["median"]=filter::median;
+    m_filterMap["savgolay"]=filter::savgolay;
   }
 
 
@@ -140,7 +148,8 @@ private:
   std::vector<double> m_taps;
   std::vector<short> m_class;
   std::vector<short> m_mask;
-   std::string m_padding;
+  std::string m_padding;
+  std::vector<double> m_noDataValues;
 };
 
 
@@ -417,14 +426,24 @@ template<class T> void Filter::applyFwhm(const std::vector<double> &wavelengthIn
   filter(input,output);
  }
 
+  template<class T> void Filter::smoothNoData(const std::vector<T>& input, const std::string& interpolationType, std::vector<T>& output)
+{
+  statfactory::StatFactory stat;
+  stat.setNoDataValues(m_noDataValues);
+  std::vector<double> abscis(input.size());
+  for(int i=0;i<abscis.size();++i)
+    abscis[i]=i;
+  stat.interpolateNoData(abscis,input,interpolationType,output);
+ }
+
 template<class T> void Filter::filter(const std::vector<T>& input, std::vector<T>& output)
 {
-  assert(input.size()>m_taps.size());
+  assert(input.size()>=m_taps.size());
   output.resize(input.size());
   int i=0;
   //start: extend input by padding
   for(i=0;i<m_taps.size()/2;++i){
-    //todo:introduce nodata
+    //todo:introduce nodata?
     output[i]=m_taps[m_taps.size()/2]*input[i];
     for(int t=1;t<=m_taps.size()/2;++t){
       output[i]+=m_taps[m_taps.size()/2+t]*input[i+t];
@@ -460,9 +479,9 @@ template<class T> void Filter::filter(const std::vector<T>& input, std::vector<T
   }
   //end: extend input by padding
   for(i=input.size()-m_taps.size()/2;i<input.size();++i){
-    //todo:introduce nodata
+    //todo:introduce nodata?
     output[i]=m_taps[m_taps.size()/2]*input[i];
-    //todo:introduce nodata
+    //todo:introduce nodata?
     for(int t=1;t<=m_taps.size()/2;++t){
       output[i]+=m_taps[m_taps.size()/2-t]*input[i-t];
       if(i+t<input.size())
@@ -497,6 +516,7 @@ template<class T> void Filter::filter(const std::vector<T>& input, std::vector<T
   output.resize(input.size());
   int i=0;
   statfactory::StatFactory stat;
+  stat.setNoDataValues(m_noDataValues);
   std::vector<T> statBuffer;
   short binValue=0;
   //start: extend input by padding
@@ -725,7 +745,7 @@ template<class T> void Filter::filter(const std::vector<T>& input, std::vector<T
 
 template<class T> void Filter::filter(T* input, int inputSize, std::vector<T>& output)
 {
-  assert(inputSize>m_taps.size());
+  assert(inputSize>=m_taps.size());
   output.resize(inputSize);
   int i=0;
 
