@@ -46,15 +46,15 @@ int main(int argc, char *argv[])
   Optionpk<bool> random_opt("random", "random", "in case of balance, randomize input data", true,2);
   Optionpk<int> minSize_opt("min", "min", "if number of training pixels is less then min, do not take this class into account (0: consider all classes)", 0);
   Optionpk<short> band_opt("b", "band", "band index (starting from 0, either use band option or use start to end)");
-  Optionpk<double> bstart_opt("bs", "bstart", "start band sequence number",0); 
-  Optionpk<double> bend_opt("be", "bend", "end band sequence number (set to 0 to include bands)", 0); 
+  Optionpk<double> bstart_opt("s", "start", "start band sequence number",0); 
+  Optionpk<double> bend_opt("e", "end", "end band sequence number (set to 0 to include bands)", 0); 
   Optionpk<double> offset_opt("\0", "offset", "offset value for each spectral band input features: refl[band]=(DN[band]-offset[band])/scale[band]", 0.0);
   Optionpk<double> scale_opt("\0", "scale", "scale value for each spectral band input features: refl=(DN[band]-offset[band])/scale[band] (use 0 if scale min and max in each band to -1.0 and 1.0)", 0.0);
   Optionpk<unsigned short> aggreg_opt("a", "aggreg", "how to combine aggregated classifiers, see also rc option (1: sum rule, 2: max rule).",1);
-  Optionpk<double> priors_opt("p", "prior", "prior probabilities for each class (e.g., -p 0.3 -p 0.3 -p 0.2 )", 0.0); 
+  Optionpk<double> priors_opt("prior", "prior", "prior probabilities for each class (e.g., -p 0.3 -p 0.3 -p 0.2 )", 0.0); 
   Optionpk<string> priorimg_opt("pim", "priorimg", "prior probability image (multi-band img with band for each class","",2); 
   Optionpk<unsigned short> cv_opt("cv", "cv", "n-fold cross validation mode",0);
-  Optionpk<unsigned int> nneuron_opt("n", "nneuron", "number of neurons in hidden layers in neural network (multiple hidden layers are set by defining multiple number of neurons: -n 15 -n 1, default is one hidden layer with 5 neurons)", 5); 
+  Optionpk<unsigned int> nneuron_opt("nn", "nneuron", "number of neurons in hidden layers in neural network (multiple hidden layers are set by defining multiple number of neurons: -n 15 -n 1, default is one hidden layer with 5 neurons)", 5); 
   Optionpk<float> connection_opt("\0", "connection", "connection reate (default: 1.0 for a fully connected network)", 1.0); 
   Optionpk<float> weights_opt("w", "weights", "weights for neural network. Apply to fully connected network only, starting from first input neuron to last output neuron, including the bias neurons (last neuron in each but last layer)", 0.0); 
   Optionpk<float> learning_opt("l", "learning", "learning rate (default: 0.7)", 0.7); 
@@ -610,10 +610,8 @@ int main(int argc, char *argv[])
     if(mask_opt.size()){
       try{
         if(verbose_opt[0]>=1)
-          cout << "opening mask image file " << mask_opt[0] << endl;
+          std::cout << "opening mask image file " << mask_opt[0] << std::endl;
         maskReader.open(mask_opt[0]);
-        assert(maskReader.nrOfCol()==testImage.nrOfCol());
-        assert(maskReader.nrOfRow()==testImage.nrOfRow());
       }
       catch(string error){
         cerr << error << endl;
@@ -745,20 +743,6 @@ int main(int argc, char *argv[])
       assert(nband==hpixel[0].size());
       if(verbose_opt[0]==2)
         cout << "used bands: " << nband << endl;
-      //read mask
-      if(!lineMask.empty()){
-        try{
-          maskReader.readData(lineMask,GDT_Int16,iline);
-        }
-        catch(string theError){
-          cerr << "Error reading " << mask_opt[0] << ": " << theError << endl;
-          exit(3);
-        }
-        catch(...){
-          cerr << "error catched" << endl;
-          exit(3);
-        }
-      }
       //read prior
       if(priorimg_opt.size()){
         try{
@@ -777,40 +761,70 @@ int main(int argc, char *argv[])
           exit(3);
         }
       }
-    
+      double oldRowMask=-1;//keep track of row mask to optimize number of line readings
       //process per pixel
       for(int icol=0;icol<ncol;++icol){
         assert(hpixel[icol].size()==nband);
         bool masked=false;
-        if(!lineMask.empty()){
-          short theMask=0;
-          for(short ivalue=0;ivalue<msknodata_opt.size();++ivalue){
-            if(msknodata_opt[ivalue]>=0){//values set in msknodata_opt are invalid
-              if(lineMask[icol]==msknodata_opt[ivalue]){
-                theMask=lineMask[icol];
-                masked=true;
-                break;
-              }
-            }
-            else{//only values set in msknodata_opt are valid
-              if(lineMask[icol]!=-msknodata_opt[ivalue]){
-                  theMask=(nodata_opt.size()==msknodata_opt.size())? nodata_opt[ivalue] : nodata_opt[0];// lineMask[icol];
-                masked=true;
-              }
-              else{
-                masked=false;
-                break;
-              }
-            }
-          }
-          if(masked){
-            if(classBag_opt.size())
-              for(int ibag=0;ibag<nbag;++ibag)
-                classBag[ibag][icol]=theMask;
-            classOut[icol]=theMask;
-            continue;
-          }
-        }
+        if(mask_opt.size()){
+	  //read mask
+	  double colMask=0;
+	  double rowMask=0;
+	  double geox=0;
+	  double geoy=0;
+
+	  testImage.image2geo(icol,iline,geox,geoy);
+	  maskReader.geo2image(geox,geoy,colMask,rowMask);
+	  colMask=static_cast<int>(colMask);
+	  rowMask=static_cast<int>(rowMask);
+	  if(rowMask>=0&&rowMask<maskReader.nrOfRow()&&colMask>=0&&colMask<maskReader.nrOfCol()){
+	    if(static_cast<int>(rowMask)!=static_cast<int>(oldRowMask)){
+	      assert(rowMask>=0&&rowMask<maskReader.nrOfRow());
+	      try{
+		// maskReader.readData(lineMask[imask],GDT_Int32,static_cast<int>(rowMask));
+		maskReader.readData(lineMask,GDT_Int16,static_cast<int>(rowMask));
+	      }
+	      catch(string errorstring){
+		cerr << errorstring << endl;
+		exit(1);
+	      }
+	      catch(...){
+		cerr << "error catched" << std::endl;
+		exit(3);
+	      }
+	      oldRowMask=rowMask;
+	    }
+	  }
+	  else
+	    continue;//no coverage in this mask
+	  short theMask=0;
+	  for(short ivalue=0;ivalue<msknodata_opt.size();++ivalue){
+	    if(msknodata_opt[ivalue]>=0){//values set in msknodata_opt are invalid
+	      if(lineMask[colMask]==msknodata_opt[ivalue]){
+		theMask=lineMask[colMask];
+		masked=true;
+		break;
+	      }
+	    }
+	    else{//only values set in msknodata_opt are valid
+	      if(lineMask[colMask]!=-msknodata_opt[ivalue]){
+		theMask=lineMask[colMask];
+		masked=true;
+	      }
+	      else{
+		masked=false;
+		break;
+	      }
+	    }
+	  }
+	  if(masked){
+	    if(classBag_opt.size())
+	      for(int ibag=0;ibag<nbag;++ibag)
+		classBag[ibag][icol]=theMask;
+	    classOut[icol]=theMask;
+	    continue;
+	  }
+	}
         bool valid=false;
         for(int iband=0;iband<nband;++iband){
           if(hpixel[icol][iband]){
