@@ -54,7 +54,7 @@ int main(int argc, char *argv[])
   Optionpk<string> ftype_opt("ft", "ftype", "Field type (only Real or Integer)", "Real");
   Optionpk<string> ltype_opt("lt", "ltype", "Label type: In16 or String", "Integer");
   Optionpk<bool> polygon_opt("polygon", "polygon", "Create OGRPolygon as geometry instead of OGRPoint.", false);
-  Optionpk<int> band_opt("b", "band", "Band index(es) to extract. Leave empty to use all bands");
+  Optionpk<int> band_opt("b", "band", "Band index(es) to extract (0 based). Leave empty to use all bands");
   Optionpk<string> rule_opt("r", "rule", "Rule how to report image information per feature (only for vector sample). point (value at each point or at centroid if polygon), centroid, mean, stdev, median, proportion, min, max, mode, sum.", "centroid");
   Optionpk<double> srcnodata_opt("srcnodata", "srcnodata", "Invalid value(s) for input image");
   Optionpk<int> bndnodata_opt("bndnodata", "bndnodata", "Band(s) in input image to check if pixel is valid (used for srcnodata)", 0);
@@ -131,6 +131,7 @@ int main(int argc, char *argv[])
   if(verbose_opt[0])
     std::cout << class_opt << std::endl;
   statfactory::StatFactory stat;
+  stat.setNoDataValues(srcnodata_opt);
   Vector2d<unsigned int> posdata;
   unsigned long int nsample=0;
   unsigned long int ntotalvalid=0;
@@ -392,12 +393,16 @@ int main(int argc, char *argv[])
                 imgReader.readData(imgBuffer[iband],GDT_Float64,static_cast<int>(jimg),theBand);
                 assert(imgBuffer[iband].size()==imgReader.nrOfCol());
 		if(srcnodata_opt.size()){
-		  vector<int>::const_iterator bndit=find(bndnodata_opt.begin(),bndnodata_opt.end(),theBand);
-		  if(bndit!=bndnodata_opt.end()){
-		    vector<int>::const_iterator bndit=find(bndnodata_opt.begin(),bndnodata_opt.end(),theBand);
-		    if(bndit!=bndnodata_opt.end()){
-		      if(imgBuffer[iband][static_cast<int>(iimg)]==srcnodata_opt[theBand])
-			valid=false;
+		  vector<int>::const_iterator bndit=bndnodata_opt.begin();
+		  vector<double>::const_iterator srcit=srcnodata_opt.begin();
+		  while(bndit!=bndnodata_opt.end()&&srcit!=srcnodata_opt.end()){
+		    if((*bndit==theBand)&&(*srcit==imgBuffer[iband][static_cast<int>(iimg)])){
+		      valid=false;
+		      break;
+		    }
+		    else{
+		      ++bndit;
+		      ++srcit;
 		    }
 		  }
 		}
@@ -421,7 +426,7 @@ int main(int argc, char *argv[])
 		  continue;//do not select for now, go to next column
               }
               else if(nvalid.size()>processClass){//absolute value
-                if(nvalid[processClass]>-theThreshold)
+                if(nvalid[processClass]>=-theThreshold)
                   continue;//do not select any more pixels for this class, go to next column to search for other classes
               }
 	      writeBuffer.push_back(sample);
@@ -588,11 +593,19 @@ int main(int argc, char *argv[])
 		int theBand=(band_opt.size()) ? band_opt[iband] : iband;
                 imgReader.readData(imgBuffer[iband],GDT_Float64,static_cast<int>(jimg),theBand);
                 assert(imgBuffer[iband].size()==imgReader.nrOfCol());
+
 		if(srcnodata_opt.size()){
-		  vector<int>::const_iterator bndit=find(bndnodata_opt.begin(),bndnodata_opt.end(),theBand);
-		  if(bndit!=bndnodata_opt.end()){
-		    if(imgBuffer[iband][static_cast<int>(iimg)]==srcnodata_opt[theBand])
+		  vector<int>::const_iterator bndit=bndnodata_opt.begin();
+		  vector<double>::const_iterator srcit=srcnodata_opt.begin();
+		  while(bndit!=bndnodata_opt.end()&&srcit!=srcnodata_opt.end()){
+		    if((*bndit==theBand)&&(*srcit==imgBuffer[iband][static_cast<int>(iimg)])){
 		      valid=false;
+		      break;
+		    }
+		    else{
+		      ++bndit;
+		      ++srcit;
+		    }
 		  }
 		}
               }
@@ -614,7 +627,7 @@ int main(int argc, char *argv[])
                   continue;//do not select for now, go to next column
               }
               else if(nvalid.size()>processClass){//absolute value
-                if(nvalid[processClass]>-theThreshold)
+                if(nvalid[processClass]>=-theThreshold)
                   continue;//do not select any more pixels for this class, go to next column to search for other classes
               }
               writeBuffer.push_back(sample);
@@ -812,7 +825,7 @@ int main(int argc, char *argv[])
 	  }
 	}
 	else{//absolute value
-	  if(ntotalvalid>-threshold_opt[0]){
+	  if(ntotalvalid>=-threshold_opt[0]){
 	    if(test_opt.size())
 	      writeTest=true;
 	    else
@@ -1025,12 +1038,17 @@ int main(int argc, char *argv[])
 		      writePointFeature = OGRFeature::CreateFeature(writeLayer->GetLayerDefn());
 		    if(verbose_opt[0]>1)
 		      std::cout << "copying fields from polygons " << std::endl;
+		    //Geometry of readFeature and writePointFeature are both wkbPoint
+		    //attributes AND geometry are copied with SetFrom
+		    //test
+		    // writePointFeature->SetGeometry(&thePoint);
 		    if(writePointFeature->SetFrom(readFeature)!= OGRERR_NONE)
 		      cerr << "writing feature failed" << std::endl;
-		    writePointFeature->SetGeometry(&thePoint);
-		    OGRGeometry *updateGeometry;
-		    updateGeometry = writePointFeature->GetGeometryRef();
-		    OGRPoint *poPoint = (OGRPoint *) updateGeometry;
+
+		    assert(wkbFlatten(writePointFeature->GetGeometryRef()->getGeometryType()) == wkbPoint);
+		    // OGRGeometry *updateGeometry;
+		    // updateGeometry = writePointFeature->GetGeometryRef();
+		    // OGRPoint *poPoint = (OGRPoint *) updateGeometry;
 		    if(verbose_opt[0]>1)
 		      std::cout << "write feature has " << writePointFeature->GetFieldCount() << " fields" << std::endl;
 		  }
@@ -1057,19 +1075,26 @@ int main(int argc, char *argv[])
 		      windowValues[iband].push_back(value);
 		    }
 		    else{
-		      if(verbose_opt[0]>1)
-			std::cout << "set field " << fieldname_opt[iband] << " to " << value << std::endl;
-		      switch( fieldType ){
-		      case OFTInteger:
-		      case OFTReal:
-			writePointFeature->SetField(fieldname_opt[iband].c_str(),value);
-			break;
-		      case OFTString:
-			writePointFeature->SetField(fieldname_opt[iband].c_str(),type2string<double>(value).c_str());
-			break;
-		      default://not supported
-			assert(0);
-			break;
+		      try{
+			if(verbose_opt[0]>1)
+			  std::cout << "set field " << fieldname_opt[iband] << " to " << value << std::endl;
+			switch( fieldType ){
+			case OFTInteger:
+			case OFTReal:
+			  writePointFeature->SetField(fieldname_opt[iband].c_str(),value);
+			  break;
+			case OFTString:
+			  writePointFeature->SetField(fieldname_opt[iband].c_str(),type2string<double>(value).c_str());
+			  break;
+			default://not supported
+			  std::string errorString="field type not supported";
+			  throw(errorString);
+			  break;
+			}
+		      }
+		      catch(std::string e){
+			std::cout << e << std::endl;
+			exit(1);
 		      }
 		    }//else
 		  }//iband
@@ -1091,7 +1116,7 @@ int main(int argc, char *argv[])
 			throw(errorString);
 		      }
 		    }
-		    //destroy feature
+ 		    //destroy feature
 		    OGRFeature::DestroyFeature( writePointFeature );
 		    ++ntotalvalid;
 		    if(verbose_opt[0])
@@ -1112,9 +1137,14 @@ int main(int argc, char *argv[])
 		// writePolygon.addRing(&writeRing);//already done
 		// writePolygon.closeRings();//already done
 		//write geometry of writePolygon
+		//test
+		// writePolygonFeature->SetGeometry(&writePolygon);
 		if(writePolygonFeature->SetFrom(readFeature)!= OGRERR_NONE)
 		  cerr << "writing feature failed" << std::endl;
+		//test
 		writePolygonFeature->SetGeometry(&writePolygon);
+		assert(wkbFlatten(writePolygonFeature->GetGeometryRef()->getGeometryType()) == wkbPolygon);
+
 		if(verbose_opt[0]>1)
 		  std::cout << "copying new fields write polygon " << std::endl;
 		if(verbose_opt[0]>1)
@@ -1125,12 +1155,17 @@ int main(int argc, char *argv[])
 		//create feature
 		if(verbose_opt[0]>1)
 		  std::cout << "copying fields from polygons " << std::endl;
+		//test
+		//Geometry of readFeature and writeCentroidFeature are both wkbPoint
+		//attributes AND geometry are copied with SetFrom
+		// writeCentroidFeature->SetGeometry(&writeCentroidPoint);
 		if(writeCentroidFeature->SetFrom(readFeature)!= OGRERR_NONE)
 		  cerr << "writing feature failed" << std::endl;
-		writeCentroidFeature->SetGeometry(&writeCentroidPoint);
-		OGRGeometry *updateGeometry;
-		updateGeometry = writeCentroidFeature->GetGeometryRef();
-		assert(wkbFlatten(updateGeometry->getGeometryType()) == wkbPoint );
+		assert(wkbFlatten(writeCentroidFeature->GetGeometryRef()->getGeometryType()) == wkbPoint);
+		//test
+		// OGRGeometry *updateGeometry;
+		// updateGeometry = writeCentroidFeature->GetGeometryRef();
+		// assert(wkbFlatten(updateGeometry->getGeometryType()) == wkbPoint );
 		if(verbose_opt[0]>1)
 		  std::cout << "write feature has " << writeCentroidFeature->GetFieldCount() << " fields" << std::endl;
 	      }
@@ -1150,72 +1185,91 @@ int main(int argc, char *argv[])
 		      std::cout << "number of points in window: " << nPointWindow << std::endl;
 		    int theBand=(band_opt.size()) ? band_opt[index] : index;
 
-		    if(verbose_opt[0]>1)
-		      std::cout << "set field " << fieldname_opt[index] << " to " << theValue << std::endl;
-		    switch( fieldType ){
-		    case OFTInteger:
-		    case OFTReal:
-		      if(polygon_opt[0])
-			writePolygonFeature->SetField(fieldname_opt[index].c_str(),theValue);
-		      else
-			writeCentroidFeature->SetField(fieldname_opt[index].c_str(),theValue);
-		      break;
-		    case OFTString:
-		      if(polygon_opt[0])
-			writePolygonFeature->SetField(fieldname_opt[index].c_str(),type2string<double>(theValue).c_str());
-		      else
-			writeCentroidFeature->SetField(fieldname_opt[index].c_str(),type2string<double>(theValue).c_str());
-		      break;
-		    default://not supported
-		      std::cout << "field type not supported yet..." << std::endl;
-		      break;
+		    try{
+		      if(verbose_opt[0]>1)
+			std::cout << "set field " << fieldname_opt[index] << " to " << theValue << std::endl;
+		      switch( fieldType ){
+		      case OFTInteger:
+		      case OFTReal:
+			if(polygon_opt[0])
+			  writePolygonFeature->SetField(fieldname_opt[index].c_str(),theValue);
+			else
+			  writeCentroidFeature->SetField(fieldname_opt[index].c_str(),theValue);
+			break;
+		      case OFTString:
+			if(polygon_opt[0])
+			  writePolygonFeature->SetField(fieldname_opt[index].c_str(),type2string<double>(theValue).c_str());
+			else
+			  writeCentroidFeature->SetField(fieldname_opt[index].c_str(),type2string<double>(theValue).c_str());
+			break;
+		      default://not supported
+			std::string errorString="field type not supported";
+			throw(errorString);
+			break;
+		      }
+		    }
+		    catch(std::string e){
+		      std::cout << e << std::endl;
+		      exit(1);
 		    }
 		  }
 		}
 		else{//ruleMap[rule_opt[0]] is not rule::point
 		  double theValue=0;
 		  for(int index=0;index<windowValues.size();++index){
-		    if(windowValues[index].size()!=buffer_opt[0]*buffer_opt[0]){
-		      cerr << "Error: windowValues[index].size()=" << windowValues[index].size() << endl;
+		    try{
+		      if(windowValues[index].size()!=buffer_opt[0]*buffer_opt[0]){
+			std::string errorString="windowValues[index].size()!=buffer*buffer";
+			throw(errorString);
+		      }
+		      if(ruleMap[rule_opt[0]]==rule::mean)
+			theValue=stat.mean(windowValues[index]);
+		      else if(ruleMap[rule_opt[0]]==rule::stdev)
+			theValue=sqrt(stat.var(windowValues[index]));
+		      else if(ruleMap[rule_opt[0]]==rule::median)
+			theValue=stat.median(windowValues[index]);
+		      else if(ruleMap[rule_opt[0]]==rule::sum)
+			theValue=stat.sum(windowValues[index]);
+		      else if(ruleMap[rule_opt[0]]==rule::max)
+			theValue=stat.mymax(windowValues[index]);
+		      else if(ruleMap[rule_opt[0]]==rule::min)
+			theValue=stat.mymin(windowValues[index]);
+		      else if(ruleMap[rule_opt[0]]==rule::centroid){
+			if(verbose_opt[0])
+			  std::cout << "number of points in polygon: " << nPointWindow << std::endl;
+			assert(nPointWindow<=1);
+			assert(nPointWindow==windowValues[index].size());
+			theValue=windowValues[index].back();
+		      }
+		      else{
+			std::string errorString="rule not supported";
+			throw(errorString);
+		      }
+		      if(verbose_opt[0]>1)
+			std::cout << "set field " << fieldname_opt[index] << " to " << theValue << std::endl;
+		      switch( fieldType ){
+		      case OFTInteger:
+		      case OFTReal:
+			if(polygon_opt[0])
+			  writePolygonFeature->SetField(fieldname_opt[index].c_str(),theValue);
+			else
+			  writeCentroidFeature->SetField(fieldname_opt[index].c_str(),theValue);
+			break;
+		      case OFTString:
+			if(polygon_opt[0])
+			  writePolygonFeature->SetField(fieldname_opt[index].c_str(),type2string<double>(theValue).c_str());
+			else
+			  writeCentroidFeature->SetField(fieldname_opt[index].c_str(),type2string<double>(theValue).c_str());
+			break;
+		      default://not supported
+			std::string errorString="field type not supported";
+			throw(errorString);
+			break;
+		      }
 		    }
-		    if(ruleMap[rule_opt[0]]==rule::mean)
-		      theValue=stat.mean(windowValues[index]);
-		    else if(ruleMap[rule_opt[0]]==rule::stdev)
-		      theValue=sqrt(stat.var(windowValues[index]));
-		    else if(ruleMap[rule_opt[0]]==rule::median)
-		      theValue=stat.median(windowValues[index]);
-		    else if(ruleMap[rule_opt[0]]==rule::sum)
-		      theValue=stat.sum(windowValues[index]);
-		    else if(ruleMap[rule_opt[0]]==rule::max)
-		      theValue=stat.mymax(windowValues[index]);
-		    else if(ruleMap[rule_opt[0]]==rule::min)
-		      theValue=stat.mymin(windowValues[index]);
-		    else{//rule::centroid
-		      if(verbose_opt[0])
-			std::cout << "number of points in polygon: " << nPointWindow << std::endl;
-		      assert(nPointWindow<=1);
-		      assert(nPointWindow==windowValues[index].size());
-		      theValue=windowValues[index].back();
-		    }
-		    if(verbose_opt[0]>1)
-		      std::cout << "set field " << fieldname_opt[index] << " to " << theValue << std::endl;
-		    switch( fieldType ){
-		    case OFTInteger:
-		    case OFTReal:
-		      if(polygon_opt[0])
-			writePolygonFeature->SetField(fieldname_opt[index].c_str(),theValue);
-		      else
-			writeCentroidFeature->SetField(fieldname_opt[index].c_str(),theValue);
-		      break;
-		    case OFTString:
-		      if(polygon_opt[0])
-			writePolygonFeature->SetField(fieldname_opt[index].c_str(),type2string<double>(theValue).c_str());
-		      else
-			writeCentroidFeature->SetField(fieldname_opt[index].c_str(),type2string<double>(theValue).c_str());
-		      break;
-		    default://not supported
-		      std::cout << "field type not supported yet..." << std::endl;
-		      break;
+		    catch(std::string e){
+		      std::cout << e << std::endl;
+		      exit(1);
 		    }
 		  }
 		}
@@ -1475,6 +1529,8 @@ int main(int argc, char *argv[])
 		  validFeature=true;
 
 		writeRing.addPoint(&thePoint);//todo: check if I need to add all interior points to ring or do I need to check if point is on ring first?
+		// if(writeRing.isPointOnRingBoundary(&thePoint))
+		//    writeRing.addPoint(&thePoint);
 		if(verbose_opt[0]>1)
 		  std::cout << "point is on surface:" << thePoint.getX() << "," << thePoint.getY() << std::endl;
 		++nPointPolygon;
@@ -1495,10 +1551,15 @@ int main(int argc, char *argv[])
 		      std::cout << "copying fields from polygons " << std::endl;
 		    if(writePointFeature->SetFrom(readFeature)!= OGRERR_NONE)
 		      cerr << "writing feature failed" << std::endl;
+		    if(verbose_opt[0]>1)
+		      std::cout << "set geometry as point " << std::endl;
+		    //test
 		    writePointFeature->SetGeometry(&thePoint);
-		    OGRGeometry *updateGeometry;
-		    updateGeometry = writePointFeature->GetGeometryRef();
-		    OGRPoint *poPoint = (OGRPoint *) updateGeometry;
+		    assert(wkbFlatten(writePointFeature->GetGeometryRef()->getGeometryType()) == wkbPoint);
+		    //test
+		    // OGRGeometry *updateGeometry;
+		    // updateGeometry = writePointFeature->GetGeometryRef();
+		    // OGRPoint *poPoint = (OGRPoint *) updateGeometry;
 		    if(verbose_opt[0]>1)
 		      std::cout << "write feature has " << writePointFeature->GetFieldCount() << " fields" << std::endl;
 		  }
@@ -1579,9 +1640,11 @@ int main(int argc, char *argv[])
 		writePolygon.addRing(&writeRing);
 		writePolygon.closeRings();
 		//write geometry of writePolygon
+		//test
+		//writePolygonFeature and readFeature are both of type wkbPolygon
+		// writePolygonFeature->SetGeometry(&writePolygon);
 		if(writePolygonFeature->SetFrom(readFeature)!= OGRERR_NONE)
 		  cerr << "writing feature failed" << std::endl;
-		writePolygonFeature->SetGeometry(&writePolygon);
 		if(verbose_opt[0]>1)
 		  std::cout << "copying new fields write polygon " << std::endl;
 		if(verbose_opt[0]>1)
@@ -1592,12 +1655,16 @@ int main(int argc, char *argv[])
 		//create feature
 		if(verbose_opt[0]>1)
 		  std::cout << "copying fields from polygons " << std::endl;
+		//test
+		//writeCentroidFeature->SetGeometry(&writeCentroidPoint);
 		if(writeCentroidFeature->SetFrom(readFeature)!= OGRERR_NONE)
 		  cerr << "writing feature failed" << std::endl;
+		//test
 		writeCentroidFeature->SetGeometry(&writeCentroidPoint);
-		OGRGeometry *updateGeometry;
-		updateGeometry = writeCentroidFeature->GetGeometryRef();
-		assert(wkbFlatten(updateGeometry->getGeometryType()) == wkbPoint );
+		assert(wkbFlatten(writeCentroidFeature->GetGeometryRef()->getGeometryType()) == wkbPoint );
+		// OGRGeometry *updateGeometry;
+		// updateGeometry = writeCentroidFeature->GetGeometryRef();
+		// assert(wkbFlatten(updateGeometry->getGeometryType()) == wkbPoint );
 		if(verbose_opt[0]>1)
 		  std::cout << "write feature has " << writeCentroidFeature->GetFieldCount() << " fields" << std::endl;
 	      }
@@ -1606,77 +1673,93 @@ int main(int argc, char *argv[])
 		  if(verbose_opt[0])
 		    std::cout << "number of points in polygon: " << nPointPolygon << std::endl;
 		  for(int index=0;index<polyValues.size();++index){
-		    //test
 		    assert(polyValues[index].size()==1);
 		    double theValue=polyValues[index].back();
 
 		    if(verbose_opt[0])
 		      std::cout << "number of points in polygon: " << nPointPolygon << std::endl;
 		    int theBand=(band_opt.size()) ? band_opt[index] : index;
-
-		    if(verbose_opt[0]>1)
-		      std::cout << "set field " << fieldname_opt[index] << " to " << theValue << std::endl;
-		    switch( fieldType ){
-		    case OFTInteger:
-		    case OFTReal:
-		      if(polygon_opt[0])
-			writePolygonFeature->SetField(fieldname_opt[index].c_str(),theValue);
-		      else
-			writeCentroidFeature->SetField(fieldname_opt[index].c_str(),theValue);
-		      break;
-		    case OFTString:
-		      if(polygon_opt[0])
-			writePolygonFeature->SetField(fieldname_opt[index].c_str(),type2string<double>(theValue).c_str());
-		      else
-			writeCentroidFeature->SetField(fieldname_opt[index].c_str(),type2string<double>(theValue).c_str());
-		      break;
-		    default://not supported
-		      std::cout << "field type not supported yet..." << std::endl;
-		      break;
+		    try{
+		      if(verbose_opt[0]>1)
+			std::cout << "set field " << fieldname_opt[index] << " to " << theValue << std::endl;
+		      switch( fieldType ){
+		      case OFTInteger:
+		      case OFTReal:
+			if(polygon_opt[0])
+			  writePolygonFeature->SetField(fieldname_opt[index].c_str(),theValue);
+			else
+			  writeCentroidFeature->SetField(fieldname_opt[index].c_str(),theValue);
+			break;
+		      case OFTString:
+			if(polygon_opt[0])
+			  writePolygonFeature->SetField(fieldname_opt[index].c_str(),type2string<double>(theValue).c_str());
+			else
+			  writeCentroidFeature->SetField(fieldname_opt[index].c_str(),type2string<double>(theValue).c_str());
+			break;
+		      default://not supported
+			std::string errorString="field type not supported";
+			throw(errorString);
+			break;
+		      }
+		    }
+		    catch(std::string e){
+		      std::cout << e << std::endl;
+		      exit(1);
 		    }
 		  }
 		}
 		else{//ruleMap[rule_opt[0]] is not rule::point
 		  double theValue=0;
 		  for(int index=0;index<polyValues.size();++index){
-		    if(ruleMap[rule_opt[0]]==rule::mean)
-		      theValue=stat.mean(polyValues[index]);
-		    else if(ruleMap[rule_opt[0]]==rule::stdev)
-		      theValue=sqrt(stat.var(polyValues[index]));
-		    else if(ruleMap[rule_opt[0]]==rule::median)
-		      theValue=stat.median(polyValues[index]);
-		    else if(ruleMap[rule_opt[0]]==rule::sum)
-		      theValue=stat.sum(polyValues[index]);
-		    else if(ruleMap[rule_opt[0]]==rule::max)
-		      theValue=stat.mymax(polyValues[index]);
-		    else if(ruleMap[rule_opt[0]]==rule::min)
-		      theValue=stat.mymin(polyValues[index]);
-		    else{//rule::centroid
-		      if(verbose_opt[0])
-			std::cout << "number of points in polygon: " << nPointPolygon << std::endl;
-		      assert(nPointPolygon<=1);
-		      assert(nPointPolygon==polyValues[index].size());
-		      theValue=polyValues[index].back();
+		    try{
+		      if(ruleMap[rule_opt[0]]==rule::mean)
+			theValue=stat.mean(polyValues[index]);
+		      else if(ruleMap[rule_opt[0]]==rule::stdev)
+			theValue=sqrt(stat.var(polyValues[index]));
+		      else if(ruleMap[rule_opt[0]]==rule::median)
+			theValue=stat.median(polyValues[index]);
+		      else if(ruleMap[rule_opt[0]]==rule::sum)
+			theValue=stat.sum(polyValues[index]);
+		      else if(ruleMap[rule_opt[0]]==rule::max)
+			theValue=stat.mymax(polyValues[index]);
+		      else if(ruleMap[rule_opt[0]]==rule::min)
+			theValue=stat.mymin(polyValues[index]);
+		      else if(ruleMap[rule_opt[0]]==rule::centroid){
+			if(verbose_opt[0])
+			  std::cout << "number of points in polygon: " << nPointPolygon << std::endl;
+			assert(nPointPolygon<=1);
+			assert(nPointPolygon==polyValues[index].size());
+			theValue=polyValues[index].back();
+		      }
+		      else{
+			std::string errorString="rule not supported";
+			throw(errorString);
+		      }
+		      if(verbose_opt[0]>1)
+			std::cout << "set field " << fieldname_opt[index] << " to " << theValue << std::endl;
+		      switch( fieldType ){
+		      case OFTInteger:
+		      case OFTReal:
+			if(polygon_opt[0])
+			  writePolygonFeature->SetField(fieldname_opt[index].c_str(),theValue);
+			else
+			  writeCentroidFeature->SetField(fieldname_opt[index].c_str(),theValue);
+			break;
+		      case OFTString:
+			if(polygon_opt[0])
+			  writePolygonFeature->SetField(fieldname_opt[index].c_str(),type2string<double>(theValue).c_str());
+			else
+			  writeCentroidFeature->SetField(fieldname_opt[index].c_str(),type2string<double>(theValue).c_str());
+			break;
+		      default:
+			std::string errorString="field type not supported";
+			throw(errorString);
+			break;
+		      }
 		    }
-		    if(verbose_opt[0]>1)
-		      std::cout << "set field " << fieldname_opt[index] << " to " << theValue << std::endl;
-		    switch( fieldType ){
-		    case OFTInteger:
-		    case OFTReal:
-		      if(polygon_opt[0])
-			writePolygonFeature->SetField(fieldname_opt[index].c_str(),theValue);
-		      else
-			writeCentroidFeature->SetField(fieldname_opt[index].c_str(),theValue);
-		      break;
-		    case OFTString:
-		      if(polygon_opt[0])
-			writePolygonFeature->SetField(fieldname_opt[index].c_str(),type2string<double>(theValue).c_str());
-		      else
-			writeCentroidFeature->SetField(fieldname_opt[index].c_str(),type2string<double>(theValue).c_str());
-		      break;
-		    default://not supported
-		      std::cout << "field type not supported yet..." << std::endl;
-		      break;
+		    catch(std::string e){
+		      std::cout << e << std::endl;
+		      exit(1);
 		    }
 		  }
 		}
@@ -1931,6 +2014,8 @@ int main(int argc, char *argv[])
 		  validFeature=true;
 		
 		writeRing.addPoint(&thePoint);
+		// if(writeRing.isPointOnRingBoundary(&thePoint))
+		//    writeRing.addPoint(&thePoint);
 		if(verbose_opt[0]>1)
 		    std::cout << "point is on surface:" << thePoint.getX() << "," << thePoint.getY() << std::endl;
 		  ++nPointPolygon;
@@ -1949,12 +2034,16 @@ int main(int argc, char *argv[])
 			writePointFeature = OGRFeature::CreateFeature(writeLayer->GetLayerDefn());
 		      if(verbose_opt[0]>1)
 			std::cout << "copying fields from polygons " << std::endl;
+		      //test
+		      // writePointFeature->SetGeometry(&thePoint);
 		      if(writePointFeature->SetFrom(readFeature)!= OGRERR_NONE)
 			cerr << "writing feature failed" << std::endl;
+		      //test
 		      writePointFeature->SetGeometry(&thePoint);
-		      OGRGeometry *updateGeometry;
-		      updateGeometry = writePointFeature->GetGeometryRef();
-		      OGRPoint *poPoint = (OGRPoint *) updateGeometry;
+		      assert(wkbFlatten(writePointFeature->GetGeometryRef()->getGeometryType()) == wkbPoint);
+		      // OGRGeometry *updateGeometry;
+		      // updateGeometry = writePointFeature->GetGeometryRef();
+		      // OGRPoint *poPoint = (OGRPoint *) updateGeometry;
 		      if(verbose_opt[0]>1)
 			std::cout << "write feature has " << writePointFeature->GetFieldCount() << " fields" << std::endl;
 		    }
@@ -2035,8 +2124,6 @@ int main(int argc, char *argv[])
 		    std::cout << "ntotalvalid: " << ntotalvalid << std::endl;
 	      }
 	    }
-
-	    //test
 	    if(!validFeature)
 	      continue;
 	    if(polygon_opt[0]||ruleMap[rule_opt[0]]!=rule::point){
@@ -2048,25 +2135,32 @@ int main(int argc, char *argv[])
 		writePolygon.addRing(&writeRing);
 		writePolygon.closeRings();
 		//write geometry of writePolygon
+		//test
+		//writePolygon and readFeature are from geometry type wkbMultiPolygon
+		// writePolygonFeature->SetGeometry(&writePolygon);
 		if(writePolygonFeature->SetFrom(readFeature)!= OGRERR_NONE)
 		  cerr << "writing feature failed" << std::endl;
-		writePolygonFeature->SetGeometry(&writePolygon);
+		assert(writePolygonFeature->GetGeometryRef()->getGeometryType()==wkbMultiPolygon);
 		if(verbose_opt[0]>1)
 		  std::cout << "copying new fields write polygon " << std::endl;
 		if(verbose_opt[0]>1)
 		  std::cout << "write feature has " << writePolygonFeature->GetFieldCount() << " fields" << std::endl;
 		//write polygon feature
 	      }
-	      else{//write mean /median value of polygon to centroid point (ruleMap[rule_opt[0]]==rule::mean, stdev or median )
+	      else{//write band information of polygon to centroid point
 		//create feature
 		if(verbose_opt[0]>1)
 		  std::cout << "copying fields from polygons " << std::endl;
+		//test
+		// writeCentroidFeature->SetGeometry(&writeCentroidPoint);
 		if(writeCentroidFeature->SetFrom(readFeature)!= OGRERR_NONE)
 		  cerr << "writing feature failed" << std::endl;
 		writeCentroidFeature->SetGeometry(&writeCentroidPoint);
-		OGRGeometry *updateGeometry;
-		updateGeometry = writeCentroidFeature->GetGeometryRef();
-		assert(wkbFlatten(updateGeometry->getGeometryType()) == wkbPoint );
+		assert(wkbFlatten(writeCentroidFeature->GetGeometryRef()->getGeometryType()) == wkbPoint);
+		//test
+		// OGRGeometry *updateGeometry;
+		// updateGeometry = writeCentroidFeature->GetGeometryRef();
+		// assert(wkbFlatten(updateGeometry->getGeometryType()) == wkbPoint );
 		if(verbose_opt[0]>1)
 		  std::cout << "write feature has " << writeCentroidFeature->GetFieldCount() << " fields" << std::endl;
 	      }
@@ -2081,120 +2175,87 @@ int main(int argc, char *argv[])
 		    if(verbose_opt[0])
 		      std::cout << "number of points in polygon: " << nPointPolygon << std::endl;
 		    int theBand=(band_opt.size()) ? band_opt[index] : index;
-
-		    if(verbose_opt[0]>1)
-		      std::cout << "set field " << fieldname_opt[index] << " to " << theValue << std::endl;
-		    switch( fieldType ){
-		    case OFTInteger:
-		    case OFTReal:
-		      if(polygon_opt[0])
-			writePolygonFeature->SetField(fieldname_opt[index].c_str(),theValue);
-		      else
-			writeCentroidFeature->SetField(fieldname_opt[index].c_str(),theValue);
-		      break;
-		    case OFTString:
-		      if(polygon_opt[0])
-			writePolygonFeature->SetField(fieldname_opt[index].c_str(),type2string<double>(theValue).c_str());
-		      else
-			writeCentroidFeature->SetField(fieldname_opt[index].c_str(),type2string<double>(theValue).c_str());
-		      break;
-		      // case OFTRealList:{
-		      //   int fieldIndex;
-		      //   int nCount;
-		      //   const double *theList;
-		      //   vector<double> vectorList;
-		      //   if(polygon_opt[0]){
-		      //     fieldIndex=writePolygonFeature->GetFieldIndex(fieldname_opt[index].c_str());
-		      //     theList=writePolygonFeature->GetFieldAsDoubleList(fieldIndex,&nCount);
-		      //     vectorList.resize(nCount+1);
-		      //     for(int index=0;index<nCount;++index)
-		      // 	vectorList[index]=theList[index];
-		      //     vectorList[nCount]=theValue;
-		      //     writePolygonFeature->SetField(fieldIndex,vectorList.size(),&(vectorList[0]));
-		      //   }
-		      //   else{
-		      //     fieldIndex=writeCentroidFeature->GetFieldIndex(fieldname_opt[index].c_str());
-		      //     theList=writeCentroidFeature->GetFieldAsDoubleList(fieldIndex,&nCount);
-		      //     vectorList.resize(nCount+1);
-		      //     for(int index=0;index<nCount;++index)
-		      // 	vectorList[index]=theList[index];
-		      //     vectorList[nCount]=theValue;
-		      //     writeCentroidFeature->SetField(fieldIndex,vectorList.size(),&(vectorList[0]));
-		      //   }
-		      //   break;
-		      // }
-		    default://not supported
-		      std::cout << "field type not supported yet..." << std::endl;
-		      break;
+		    try{
+		      if(verbose_opt[0]>1)
+			std::cout << "set field " << fieldname_opt[index] << " to " << theValue << std::endl;
+		      switch( fieldType ){
+		      case OFTInteger:
+		      case OFTReal:
+			if(polygon_opt[0])
+			  writePolygonFeature->SetField(fieldname_opt[index].c_str(),theValue);
+			else
+			  writeCentroidFeature->SetField(fieldname_opt[index].c_str(),theValue);
+			break;
+		      case OFTString:
+			if(polygon_opt[0])
+			  writePolygonFeature->SetField(fieldname_opt[index].c_str(),type2string<double>(theValue).c_str());
+			else
+			  writeCentroidFeature->SetField(fieldname_opt[index].c_str(),type2string<double>(theValue).c_str());
+			break;
+		      default://not supported
+			std::string errorString="field type not supported";
+			throw(errorString);
+			break;
+		      }
+		    }
+		    catch(std::string e){
+		      std::cout << e << std::endl;
+		      exit(1);
 		    }
 		  }
 		}
 		else{//ruleMap[rule_opt[0]] is not rule::point
 		  double theValue=0;
 		  for(int index=0;index<polyValues.size();++index){
-		    if(ruleMap[rule_opt[0]]==rule::mean)
-		      theValue=stat.mean(polyValues[index]);
-		    else if(ruleMap[rule_opt[0]]==rule::stdev)
-		      theValue=sqrt(stat.var(polyValues[index]));
-		    else if(ruleMap[rule_opt[0]]==rule::median)
-		      theValue=stat.median(polyValues[index]);
-		    else if(ruleMap[rule_opt[0]]==rule::sum)
-		      theValue=stat.sum(polyValues[index]);
-		    else if(ruleMap[rule_opt[0]]==rule::max)
-		      theValue=stat.mymax(polyValues[index]);
-		    else if(ruleMap[rule_opt[0]]==rule::min)
-		      theValue=stat.mymin(polyValues[index]);
-		    else{//rule::centroid
-		      if(verbose_opt[0])
-			std::cout << "number of points in polygon: " << nPointPolygon << std::endl;
-		      assert(nPointPolygon<=1);
-		      assert(nPointPolygon==polyValues[index].size());
-		      theValue=polyValues[index].back();
+		    try{
+		      if(ruleMap[rule_opt[0]]==rule::mean)
+			theValue=stat.mean(polyValues[index]);
+		      else if(ruleMap[rule_opt[0]]==rule::stdev)
+			theValue=sqrt(stat.var(polyValues[index]));
+		      else if(ruleMap[rule_opt[0]]==rule::median)
+			theValue=stat.median(polyValues[index]);
+		      else if(ruleMap[rule_opt[0]]==rule::sum)
+			theValue=stat.sum(polyValues[index]);
+		      else if(ruleMap[rule_opt[0]]==rule::max)
+			theValue=stat.mymax(polyValues[index]);
+		      else if(ruleMap[rule_opt[0]]==rule::min)
+			theValue=stat.mymin(polyValues[index]);
+		      else if(ruleMap[rule_opt[0]]==rule::centroid){
+			if(verbose_opt[0])
+			  std::cout << "number of points in polygon: " << nPointPolygon << std::endl;
+			assert(nPointPolygon<=1);
+			assert(nPointPolygon==polyValues[index].size());
+			theValue=polyValues[index].back();
+		      }
+		      else{
+			std::string errorString="rule not supported";
+			throw(errorString);
+		      }
+		      if(verbose_opt[0]>1)
+			std::cout << "set field " << fieldname_opt[index] << " to " << theValue << std::endl;
+		      switch( fieldType ){
+		      case OFTInteger:
+		      case OFTReal:
+			if(polygon_opt[0])
+			  writePolygonFeature->SetField(fieldname_opt[index].c_str(),theValue);
+			else
+			  writeCentroidFeature->SetField(fieldname_opt[index].c_str(),theValue);
+			break;
+		      case OFTString:
+			if(polygon_opt[0])
+			  writePolygonFeature->SetField(fieldname_opt[index].c_str(),type2string<double>(theValue).c_str());
+			else
+			  writeCentroidFeature->SetField(fieldname_opt[index].c_str(),type2string<double>(theValue).c_str());
+			break;
+		      default://not supported
+			std::string errorString="field type not supported";
+			throw(errorString);
+			break;
+		      }
 		    }
-		    if(verbose_opt[0]>1)
-		      std::cout << "set field " << fieldname_opt[index] << " to " << theValue << std::endl;
-		    switch( fieldType ){
-		    case OFTInteger:
-		    case OFTReal:
-		      if(polygon_opt[0])
-			writePolygonFeature->SetField(fieldname_opt[index].c_str(),theValue);
-		      else
-			writeCentroidFeature->SetField(fieldname_opt[index].c_str(),theValue);
-		      break;
-		    case OFTString:
-		      if(polygon_opt[0])
-			writePolygonFeature->SetField(fieldname_opt[index].c_str(),type2string<double>(theValue).c_str());
-		      else
-			writeCentroidFeature->SetField(fieldname_opt[index].c_str(),type2string<double>(theValue).c_str());
-		      break;
-		      // case OFTRealList:{
-		      // int fieldIndex;
-		      // int nCount;
-		      // const double *theList;
-		      // vector<double> vectorList;
-		      // if(polygon_opt[0]){
-		      //   fieldIndex=writePolygonFeature->GetFieldIndex(fieldname_opt[index].c_str());
-		      //   theList=writePolygonFeature->GetFieldAsDoubleList(fieldIndex,&nCount);
-		      //   vectorList.resize(nCount+1);
-		      //   for(int index=0;index<nCount;++index)
-		      // 	vectorList[index]=theList[index];
-		      //   vectorList[nCount]=theValue;
-		      //   writePolygonFeature->SetField(fieldIndex,vectorList.size(),&(vectorList[0]));
-		      // }
-		      // else{
-		      //   fieldIndex=writeCentroidFeature->GetFieldIndex(fieldname_opt[index].c_str());
-		      //   theList=writeCentroidFeature->GetFieldAsDoubleList(fieldIndex,&nCount);
-		      //   vectorList.resize(nCount+1);
-		      //   for(int index=0;index<nCount;++index)
-		      // 	vectorList[index]=theList[index];
-		      //   vectorList[nCount]=theValue;
-		      //   writeCentroidFeature->SetField(fieldIndex,vectorList.size(),&(vectorList[0]));
-		      // }
-		      // break;
-		      // }
-		    default://not supported
-		      std::cout << "field type not supported yet..." << std::endl;
-		      break;
+		    catch(std::string e){
+		      std::cout << e << std::endl;
+		      exit(1);
 		    }
 		  }
 		}
